import sys
import unittest

from waskr.plugins import Plugin  


class MockSys(object):
    """Can grab messages sent to stdout or stderr"""
    def __init__(self):
        self.message = []

    def write(self, string):
        self.message.append(string)
        pass

    def captured(self):
        return ''.join(self.message)

class TestPlugin(unittest.TestCase):

    def test_new_plugin(self):
        plugin = Plugin(name="my_plugin",action="store_true",
                description="my help menu")
        self.assertEqual(plugin.name, "my_plugin")
        self.assertEqual(plugin.action, "store_true")
        self.assertEqual(plugin.description, "my help menu")

    def test_command_dict(self):
        plugin = Plugin(name="my_plugin",action="store_true",
                description="my help menu")
        actual = type(plugin.command())
        expected = dict
        self.assertEqual(actual, expected) 

    def test_command_values(self):
        plugin = Plugin(name="my_plugin",action="store_true",
                description="my help menu")
        actual = plugin.command()
        expected = dict(
            option = "my_plugin",
            action = "store_true",
            description = "my help menu"
            )

        self.assertEqual(actual, expected) 

    def test_run_none(self):
        """Nothing should happen when run is called"""
        sys.stderr = MockSys()
        sys.stdout = MockSys()
        plugin = Plugin(name="my_plugin",action="store_true",
                description="my help menu")
        actual = plugin.run(value=None)
        expected = None
        self.assertEqual(actual, expected)  

    def test_run_message(self):
        """Display a message to stdout when run is not set properly"""
        sys.stderr = MockSys()
        sys.stdout = MockSys()
        plugin = Plugin(name="my_plugin",action="store_true",
                description="my help menu")
        plugin.run(value=None)
        actual = sys.stdout.captured() 
        expected = """
If you see this text, it means you are *not* using
the "run()" method in your plugin code.
You should be defining such a method to be executed when 
your plugin is called.

"""
        self.assertEqual(actual, expected)  


if __name__ == '__main__':
    unittest.main()



import unittest
from time import time

from waskr.middleware import RequestStatsMiddleware

class TestRequestStatsMiddleware(unittest.TestCase):

    def test_init(self):
        app = RequestStatsMiddleware(app=None)
        self.assertEqual(app.app, None)
        self.assertEqual(type(app.config), dict)

    def test_call_app_exception(self):
        app = RequestStatsMiddleware(app=None)
        app(environ=None, start_response=None)
        actual = app.parser.cache
        expected = []
        self.assertEqual(actual, expected) 

    def test_call_app(self):
        environ = {'PATH_INFO':'/'}
        app = RequestStatsMiddleware(app=None)
        app(environ=environ, start_response=None)
        actual = app.parser.cache

        self.assertEqual(type(actual), list)
        self.assertEqual(actual[0]['url'], '/')
        self.assertEqual(actual[0]['application'], 'main')
        self.assertEqual(actual[0]['server_id'], '1')
        
    def test_timing(self):
        """Timing substracts zero from current time"""
        application = RequestStatsMiddleware(app=None)
        actual = int(application.timing(zero=0))
        expected = int(time())
        self.assertEqual(actual, expected)

    def test_config(self):
        """Config should be defaults if None is passed"""
        application = RequestStatsMiddleware(app=None)
        actual = application.config 
        expected = {
                'server_id': '1', 
                'web_port': '8080', 
                'db_port': 27017, 
                'application': 'main', 
                'db_host': 'localhost', 
                'web_host': 'localhost',
                'web_user':False,
                'cache': '10',
                'plugin_path': False,
                'plugins':None,
                'log_level': 'DEBUG',
                'log_format': '%(asctime)s %(levelname)s %(name)s  %(message)s',
                'log_datefmt' : '%H:%M:%S'
                }
        self.assertEqual(actual, expected)

if __name__ == '__main__':
    unittest.main()



class MockSys(object):
    """Can grab messages sent to stdout or stderr"""
    def __init__(self):
        self.message = []

    def write(self, string):
        self.message.append(string)
        pass

    def captured(self):
        return ''.join(self.message)




#!/usr/bin/env python
#
# Copyright (c) 2009-2010 Alfredo Deza <alfredodeza [at] gmail [dot] com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import os
import sys
from optparse import OptionParser
from os import path

from waskr import database, config_options
from waskr.web import server

WARNING = """ 
     +-------------------------------------------------+
     |                 ** WARNING **                   |
     |                                                 |
     |  You have not set a configuration file for the  |
     |  command line version of waskr.                 |
     |                                                 |
     |  To add a configuration file, run:              |
     |                                                 |
     |    waskr --add-config /path/to/config           |
     |                                                 |
     +-------------------------------------------------+
"""
 

class WaskrCommands(object):
    """A lot of complicated options can happen with Pacha, so 
    it is easier if everything lives under a class rather than
    the widely used main()"""

    def __init__(self, argv=None, test=False, parse=True, db=database.conf_db()):
        if argv is None:
            argv = sys.argv

        self.test = test
        if parse:
            self.parseArgs(argv)
        self.config = {}


    def msg(self, msg, std="out"):
        if std == "out":
            sys.stdout.write(msg)
        else:
            sys.stderr.write(msg)
        if not self.test:
            sys.exit(1)


    def add_config(self, path):
        abspath = os.path.abspath(path)
        self.db.add_config(abspath)
        self.msg("Configuration file added: %s" % abspath)


    def check_config(self):
        # if any commands are run, check for a MASTER config file Location
        config_db = self.db.get_config_path()
        config_file = None
        try:
            config_list = [i for i in self.db.get_config_path()]
            config_file = config_list[0][0]
            config = config_options(config_file)
            return config
        except IndexError:
            self.msg(msg=WARNING, std="err")
 

    def config_values(self):
        try:
            config_list = [i for i in self.db.get_config_path()]
            config_file = config_list[0][0]
            config = config_options(config_file)
            print "\nConfiguration file: %s\n" % config_file
            for i in config.items():
                print "%-15s= %-4s" % (i[0], i[1])
        except Exception, error:
            print "Could not complete command: %s" % error 



    def parseArgs(self, argv):
        parser = OptionParser(description="""
A stats engine for WSGI applications
    """
        ,version='0.0.9')

        parser.add_option('--server', action='store_true', help="""Runs the web server. 
    If no configuration file is passed localhost and port 8080 is used.""")
        parser.add_option('--conf', help="""Pass a INI configuration file""")
        parser.add_option('--add-user', help="""Adds a user email for the web interface""")
        parser.add_option('--remove-user', help="""Removes a user email from the web interface""")

        options, arguments = parser.parse_args()

        if len(sys.argv) <= 1:
            parser.print_help()

   
        if options.server:
            if options.conf and path.isfile(options.conf):
                configuration = config_options(options.conf)
                server.CONF = configuration
                server.main(configuration)
            else:
                server.main()

        if options.add_user and options.conf:
            try:
                config = config_options(options.conf)
                db = database.Stats(config)
                db.add_user(options.add_user)
                print "User %s was added to the DB" % options.add_user
            except Exception, e:
                print 'waskrc could not add user: %s' % e

        if options.remove_user and options.conf:
            try:
                config = config_options(options.conf)
                db = database.Stats(config)
                db.remove_user(options.add_user)
                print "User %s removed from DB" % options.remove_user
            except Exception, e:
                print 'waskrc could not remove user: %s' % e

 

main = WaskrCommands

def main_():
    main()




import sys

class Plugin(object):

    def __init__(self, 
            name,
            action='store_true',
            description="",
            config={}):
        self.name = name
        self.action = action 
        self.description = description
        self.config = config

    def command(self):
        return dict(
            option = self.name,
            action = self.action,
            description = self.description
            )

    def run(self, value):
        """A safe way to prevent tracebacks in the CLI
        output if a dev forgets to use "run" as a method
        in their plugin"""
        print """
If you see this text, it means you are *not* using
the "run()" method in your plugin code.
You should be defining such a method to be executed when 
your plugin is called.
"""
       
    def config_values(self):
        """Provides access to the already existing configuration
        stored in waskr. Returns a dictionary of values"""
        return self.config


def load_plugins(plugins):
    for plugin in plugins:
        try:
            __import__(plugin, None, None, [''])
        except ImportError:
            pass # probably a poorly configured plugin

def init_plugin_system(cfg):
    if not cfg['plugin_path'] in sys.path:
        sys.path.insert(0, cfg['plugin_path'])
    load_plugins(cfg['plugins'])
    

def find_plugins():
    return Plugin.__subclasses__()





import sys
import unittest
import waskr
from waskr import database
from mock import MockSys

class TestWarning(unittest.TestCase):

    def test_warning(self):
        actual = waskr.WARNING
        expected = """ 
     +-------------------------------------------------+
     |                 ** WARNING **                   |
     |                                                 |
     |  You have not set a configuration file for the  |
     |  command line version of waskr.                 |
     |                                                 |
     |  To add a configuration file, run:              |
     |                                                 |
     |    waskr --add-config /path/to/config           |
     |                                                 |
     +-------------------------------------------------+
"""
        self.assertEqual(actual, expected) 

class TestWaskrCommands(unittest.TestCase):


    def test_init_(self):
        actual = waskr.WaskrCommands(argv=None, test=False, parse=False, db=False)

        self.assertEqual(actual.test, False)
        self.assertEqual(actual.db, False)

    
    def test_enable_plugins_keyerror(self):
        commands = waskr.WaskrCommands(argv=None, test=False, parse=False, db=False)
        actual = commands.enable_plugins()

        self.assertFalse(actual) 

    
    def test_enable_plugins_attributeerror(self):
        commands = waskr.WaskrCommands(argv=None, test=False, parse=False, db=False)
        commands.config = {'plugin_path':'/tmp', 'plugins':False}
        actual = commands.enable_plugins()

        self.assertFalse(actual)


    def test_enable_plugins_none(self):
        commands = waskr.WaskrCommands(argv=None, test=False, parse=False, db=False)
        commands.config = {'plugin_path':False, 'plugins':''}
        actual = commands.enable_plugins()

        self.assertFalse(actual)


    def test_enable_plugins(self):
        commands = waskr.WaskrCommands(argv=None, test=False, parse=False, db=False)
        commands.config = {'plugin_path':'/tmp', 'plugins':'foo'}
        actual = commands.enable_plugins()

        self.assertFalse(actual)


    def test_msg_stdout(self):
        sys.stderr = MockSys()
        sys.stdout = MockSys()
        commands = waskr.WaskrCommands(argv=None, test=True, parse=False, db=False)
        commands.msg("foo")
        expected = "foo"
        actual = sys.stdout.captured()
        self.assertEqual(actual, expected) 


    def test_msg_stderr(self):
        sys.stderr = MockSys()
        sys.stdout = MockSys()
        commands = waskr.WaskrCommands(argv=None, test=True, parse=False, db=False)
        commands.msg("bar", std="err")
        expected = "bar"
        actual = sys.stderr.captured()
        self.assertEqual(actual, expected) 

    
    def test_check_config_indexerror(self):
        commands = waskr.WaskrCommands(argv=None, test=True, 
                parse=False, db=database.conf_db(db=':memory:'))
        actual = commands.check_config()
        self.assertFalse(actual)
        

    def test_check_config(self):
        sys.stderr = MockSys()
        sys.stdout = MockSys()
        commands = waskr.WaskrCommands(argv=None, test=True, 
                parse=False, db=database.conf_db(db=':memory:'))
        commands.add_config('/tmp/waskr.ini')
        actual = commands.check_config()
    
        self.assertTrue(actual)
        self.assertEqual(type(actual), dict)


    def test_add_config(self):
        sys.stderr = MockSys()
        sys.stdout = MockSys()
        commands = waskr.WaskrCommands(argv=None, test=True, 
                parse=False, db=database.conf_db(db=':memory:'))
        commands.add_config('/tmp/waskr.ini')
        actual = sys.stdout.captured()
        expected = 'Configuration file added: /tmp/waskr.ini'
    
        self.assertEqual(actual, expected) 
        

    def test_config_values_error(self):
        sys.stderr = MockSys()
        commands = waskr.WaskrCommands(argv=None, test=True, 
                parse=False, db=None)
        commands.config_values()
        actual = sys.stderr.captured()
        expected = "Could not complete command: 'NoneType' object has no attribute 'get_config_path'"
        self.assertEqual(actual, expected) 


    def test_config_values(self):
        sys.stdout = MockSys()
        commands = waskr.WaskrCommands(argv=None, test=True, 
                parse=False, db=database.conf_db(db=':memory:'))
        commands.add_config('/tmp/waskr.ini')
        commands.config_values()
        actual = sys.stdout.captured()
        expected = u'Configuration file added: /tmp/waskr.ini\nConfiguration file: /tmp/waskr.ini\n\nweb_user       = False\nserver_id      = 1   \ncache          = 10  \nweb_port       = 8080\nlog_level      = DEBUG\ndb_port        = 27017\napplication    = main\ndb_host        = localhost\nplugins        = None\nlog_datefmt    = %H:%M:%S\nlog_format     = %(asctime)s %(levelname)s %(name)s  %(message)s\nplugin_path    = False\nweb_host       = localhost\n'
        self.assertEqual(actual, expected) 


    def test_clean_plugin_option_single(self):
        commands = waskr.WaskrCommands(argv=None, test=True, 
                parse=False, db=database.conf_db(db=':memory:'))
        actual = commands.clean_plugin_option("foo-bar")
        expected = "foo_bar"
        self.assertEqual(actual, expected) 


    def test_clean_plugin_option_double(self):
        commands = waskr.WaskrCommands(argv=None, test=True, 
                parse=False, db=database.conf_db(db=':memory:'))
        actual = commands.clean_plugin_option("--foo")
        expected = "foo"
        self.assertEqual(actual, expected) 


    def test_clean_plugin_option_double_single(self):
        commands = waskr.WaskrCommands(argv=None, test=True, 
                parse=False, db=database.conf_db(db=':memory:'))
        actual = commands.clean_plugin_option("--foo-bar")
        expected = "foo_bar"
        self.assertEqual(actual, expected) 

    
if __name__ == '__main__':
    unittest.main()




from waskr import util
import unittest

class TestRequestParser(unittest.TestCase):
    
    def test_init(self):
        rp = util.RequestParser(config={}, test=True)

        self.assertEqual(rp.config, {})
        self.assertEqual(rp.cache, [])

    def test_construct_no_len(self):
        rp = util.RequestParser(config={'cache':2}, test=True)
        rp.construct('foo')
        actual = rp.cache[0]
        expected = 'foo'
        self.assertEqual(actual, expected) 
        self.assertEqual(len(rp.cache), 1)

    def test_flush(self):
        rp = util.RequestParser(config={'cache':1}, test=True)
        rp.cache = ['foo']
        actual = rp.cache[0]
        expected = 'foo'
        
        self.assertEqual(actual, expected) 
        self.assertEqual(len(rp.cache), 1)

        rp.flush()

        self.assertEqual(rp.cache, [])

    



#!/usr/bin/env python
#
# Copyright (c) 2009-2010 Alfredo Deza <alfredodeza [at] gmail [dot] com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import os
import sys
from optparse import OptionParser, OptionGroup

from waskr import database
from waskr.config import options
from waskr.plugins import init_plugin_system, find_plugins

WARNING = """ 
     +-------------------------------------------------+
     |                 ** WARNING **                   |
     |                                                 |
     |  You have not set a configuration file for the  |
     |  command line version of waskr.                 |
     |                                                 |
     |  To add a configuration file, run:              |
     |                                                 |
     |    waskr --add-config /path/to/config           |
     |                                                 |
     +-------------------------------------------------+
"""


class WaskrCommands(object):
    """A lot of complicated options can happen with Pacha, so 
    it is easier if everything lives under a class rather than
    the widely used main()"""

    def __init__(self, argv=None, test=False, parse=True, db=database.conf_db()):
        self.db = db
        if argv is None:
            argv = sys.argv

        self.got_plugins = False
        self.test = test
        self.config = {}
        if parse:
            self.parseArgs(argv)

    def enable_plugins(self):
        try:
            plugin_path = self.config['plugin_path']
            plugins = self.config['plugins'].split(' ')
        except KeyError:
            return False
        except AttributeError:
            return False
        if plugin_path is False or plugins is None:
            return False
        else:
            init_plugin_system({'plugin_path': plugin_path, 'plugins': plugins})
            find_plugins()
       

    def msg(self, msg, std="out"):
        if std == "out":
            sys.stdout.write(msg)
        else:
            sys.stderr.write(msg)
        if not self.test:
            sys.exit(1)


    def add_config(self, path):
        abspath = os.path.abspath(path)
        self.db.add_config(abspath)
        self.msg("Configuration file added: %s" % abspath)


    def check_config(self):
        # if any commands are run, check for a MASTER config file Location
        config_db = self.db.get_config_path()
        config_file = None
        try:
            config_list = [i for i in self.db.get_config_path()]
            config_file = config_list[0][0]
            conf = options(config_file)
            self.config = conf
            return conf
        except IndexError:
            return False
 

    def config_values(self):
        try:
            config_list = [i for i in self.db.get_config_path()]
            config_file = config_list[0][0]
            conf = options(config_file)
            print "\nConfiguration file: %s\n" % config_file
            for i in conf.items():
                print "%-15s= %-4s" % (i[0], i[1])
        except Exception, error:
            self.msg("Could not complete command: %s" % error, std="err") 


    def clean_plugin_option(self, option):
        if '--' in option:
            option = option.replace('--', '')
        if '-' in option:
            option = option.replace('-', '_')
        return option


    def parseArgs(self, argv):
        parser = OptionParser(description="""
A stats engine for WSGI applications
    """
        ,version='0.0.9')

        parser.add_option('--server', action='store_true', help="""Runs the web server. 
 If no configuration file is passed localhost and port 8080 is used.""")
        parser.add_option('--add-config', 
                help="""Adds (overwrites) a config file to the command line app""")
        parser.add_option('--config-values', action="store_true", 
                help="""Show the current parsed config values""")

        # Plugin Group
        plugin_group = OptionGroup(parser, "Plugin Options", "Extend via\
 plugins that can run with this command line tool. If activated you should see\
 them displayed here.")

        if self.check_config() and self.enable_plugins() is not False:
            from waskr.plugins import find_plugins
            self.got_plugins = True
            plugin_options = {}
            for plugin in find_plugins():
                try:
                    cmd = plugin()
                    cmd.config = self.config
                    info = cmd.command()
                    # register it with OptParse to avoid errors
                    plugin_group.add_option(info['option'], action=info['action'],
                        help=info['description'])

                    # now register with a dict so we now what to call when we hit 
                    # that option 
                    plugin_options[info['option']] = cmd 

                except AttributeError:
                    pass # You may have not added what we need to implement
                         # your plugin

        parser.add_option_group(plugin_group)

        options, arguments = parser.parse_args()

        if options.add_config:
            self.add_config(options.add_config)

        # make sure we have a config to work with 
        if self.check_config() == False:
            self.msg(msg=WARNING, std="err")

        if len(sys.argv) <= 1:
            parser.print_help()

        if options.config_values:
            self.config_values()
   
        if options.server:
            # imports here to avoid db connection errors
            from waskr.web import server
            server.CONF = self.config
            server.main(self.config)

        # if we hit the end of *our* options, loop over the plugin options: 
        elif self.got_plugins:
            for i in plugin_options:
                if i in sys.argv[1:]:
                    obj = plugin_options[i]
                    option_value = None
                    if len(sys.argv) == 3:
                        option_value = sys.argv[2] 
                    obj.run(option_value)


main = WaskrCommands

def main_():
    main()




import distribute_setup
distribute_setup.use_setuptools()
from setuptools import setup

tests_require = ['nose', 'webtest']

setup(
    name = "waskr",
    version = "0.0.10",
    packages = ['waskr'],
    install_requires = ['bottle>=0.8', 'pymongo'],
    entry_points = {
        'console_scripts': [
            'waskr = waskr:main_'
            ]
        },
    include_package_data=True,
    package_data = {
        '': ['distribute_setup.py'],
        },

    # metadata 
    author = "Alfredo Deza",
    author_email = "alfredodeza [at] gmail [dot] com",
    description = "Stats Middleware for WSGI applications.",
    long_description = """\
 Provides statistics for any WSGI application:

  * Requests Per Second
  * Time to respond

 Data can be accessed via a web interface and/or writing your own
 plugins.
 The plugins API is simple yet flexible enough to allow you to 
 get the specific stats you may need.

 Configuration is not needed and waskr provides a simple way 
 of configuring the middleware. 
 You can pass a dictionary with some values a path to an INI file 
 or work with paster/pylons/turbogears configuration objects.

 Full documentation can be found at http://code.google.com/p/waskr
 """,

   classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
      ],

    license = "MIT",
    keywords = "WSGI stats statistics request measure performance",
    url = "http://code.google.com/p/waskr",   

)




import os
from pymongo import Connection, DESCENDING
import sqlite3
from time import time, strftime, gmtime
from waskr.config import options
import log

# Fixes Database Absolute Location
FILE_CWD =  os.path.abspath(__file__)
FILE_DIR = os.path.dirname(FILE_CWD)
DB_FILE = FILE_DIR+'/waskr.db'


class conf_db(object):

    def __init__(self,
            db = DB_FILE):
        self.db = db 
        if os.path.isfile(self.db):
            self.conn = sqlite3.connect(self.db)
            self.c = self.conn.cursor()
        else:
            self.conn = sqlite3.connect(self.db)
            table = """CREATE TABLE config(path TEXT)"""
            self.c = self.conn.cursor()
            self.c.execute(table)
            self.conn.commit()


    def closedb(self):
        """Make sure the db is closed"""
        self.conn.close()


    def add_config(self, path):
        """Adds a MASTER config for waskr"""
        values = (path,path)
        delete = 'DELETE FROM config'
        command = 'INSERT INTO config(path) select ? WHERE NOT EXISTS(SELECT 1 FROM config WHERE path=?)' 
        self.c.execute(delete)
        self.c.execute(command, values)
        self.conn.commit()


    def get_config_path(self):
        """Returns the first entry for the config path"""
        command = "SELECT * FROM config limit 1"
        return self.c.execute(command)


class Stats(object):

    def __init__(self, config=None, test=False):
        config = options(config)
        try:
            self.connection = Connection(
                    config['db_host'], 
                    int(config['db_port']))
        except Exception, error:
            log.database.critical("Could not connect to MongoDB: %s" % error)
        self.waskr = self.connection['waskr']
        self.stats = self.waskr['stats']
        self.users = self.waskr['user']
        log.database.debug("database connection initialized")
        if test:
            self.waskr = self.connection['test_waskr']
            self.stats = self.waskr['stats']
            self.users = self.waskr['user']

    def insert(self, stats):
        # Stats should come as a list of dictionaries
        for stat in stats:
            self.stats.insert(stat)
        log.database.debug("inserted stats to database")

    def last_insert(self):
        last = self.stats.find().sort('time', DESCENDING).limit(1)
        for entry in last:
            unix_time =  entry['time']
            struct = gmtime(unix_time)
            formatted = strftime('%Y-%m-%d %H:%M:%S', struct)
            return formatted
        log.model.debug("returned formatted last insert from database")

    def apps_nodes(self):
        apps = []
        for application in self.stats.distinct('application'):
            c = self.stats.find({'application':application})
            for record in c:
                match = (record['application']), record['server_id']
                if match not in apps:
                    apps.append(match)
        return apps
        log.model.debug("returned all application nodes present")

    def response_time(self, minutes):
        """Get the last N minutes of response stats"""
        time_response = []
        mins = int(minutes) * 60
        start = time() - int(mins)
        records = self.stats.find({"time": {"$gte": start, "$lt": time()}})
        for stat in records:
            data = []
            miliseconds = int(stat['time']) * 1000
            data.append(miliseconds)
            data.append(stat['response'])
            time_response.append(data)
        log.model.debug("returned the last N minutes of response stats")
        return time_response

    def response_bundle(self, minutes):
        """Get the last N minutes of response stats in a bundle"""
        time_response = []
        total = 0
        mins = int(minutes) * 60
        start = time() - int(mins)
        records = self.stats.find({"time": {"$gte": start, "$lt": time()}})
        for stat in records:
            total = total + stat['response']
            time_response.append(stat['response'])
        divide_by = len(time_response)
        try:
            average = total /divide_by
        except ZeroDivisionError:
            average = 0
        return average

    def request_bundle(self, minutes):
        """Get the last N minutes of request stats in a bundle"""
        mins = int(minutes) * 60
        start = time() - int(mins)
        hits = self.stats.find({"time": {"$gte": start, "$lt": time()}}).count()
        return hits
        log.model.debug("returned requests per second")
 
    def request_time(self, minutes):
        """Get the last N minutes of request stats"""
        requests_second = []
        mins = int(minutes) * 60
        start = time() - int(mins)
        records = self.stats.find({"time": {"$gte": start, "$lt": time()}})
        for stat in records:
            data = []
            hits = self.stats.find({'time':stat['time']}).count()
            miliseconds = int(stat['time']) * 1000
            data.append(miliseconds)
            data.append(hits)
            requests_second.append(data)
        return requests_second
        log.model.debug("returned requests per second")
    

